@@ -27,8 +27,8 @@
 (define-module (gnucash report owner-report))
 
 (use-modules (srfi srfi-1))
-(use-modules (ice-9 slib))
 (use-modules (gnucash gnc-module))
+(use-modules (gnucash printf))
 (use-modules (gnucash main))		; for gnc:debug
 
 (gnc:module-load "gnucash/report/report-system" 0)
@@ -40,11 +40,15 @@
 ;; Option names
 (define optname-from-date (N_ "From"))
 (define optname-to-date (N_ "To"))
+(define optname-date-driver (N_ "Due or Post Date"))
+
+;; let's define a name for the report-guid's, much prettier
+(define employee-report-guid "08ae9c2e884b4f9787144f47eacd7f44")
+(define vendor-report-guid "d7d1e53505ee4b1b82efad9eacedaea0")
+(define customer-report-guid "c146317be32e4948a561ec7fc89d15c1")
 
 (define acct-string (N_ "Account"))
-(define owner-string (N_ "Company"))
 (define owner-page gnc:pagename-general)
-
 (define date-header (N_ "Date"))
 (define due-date-header (N_ "Due Date"))
 (define reference-header (N_ "Reference"))
@@ -52,6 +56,36 @@
 (define desc-header (N_ "Description"))
 (define amount-header (N_ "Amount"))
 
+;; Depending on the report type we want to set up some lists/cases 
+;; with strings to ease overview and translation
+;; note: we default to company
+
+;; owner-string & doctype-str are nearly equivalent, report is vendor report
+;; but default option naming was Company. 
+
+;; Names in Option panel (Untranslated! Because it is used for option
+;; naming and lookup only, and the display of the option name will be
+;; translated somewhere else.)
+(define (owner-string owner-type)
+  (cond ((eqv? owner-type GNC-OWNER-CUSTOMER) (N_ "Customer"))
+        ((eqv? owner-type GNC-OWNER-EMPLOYEE) (N_ "Employee"))
+        ;; FALLTHROUGH
+        (else (N_ "Company")))) 
+
+;; Error strings in case there is no (valid) selection (translated)
+(define (invalid-selection-string owner-type)
+  (cond ((eqv? owner-type GNC-OWNER-CUSTOMER) (_ "No valid customer selected. Click on the Options button to select a customer."))
+        ((eqv? owner-type GNC-OWNER-EMPLOYEE) (_ "No valid employee selected. Click on the Options button to select an employee."))
+        ;; FALLTHROUGH
+        (else (_ "No valid company selected. Click on the Options button to select a company."))))
+
+;; Document names, used in report names (translated)
+(define (doctype-str owner-type)
+  (cond ((eqv? owner-type GNC-OWNER-CUSTOMER) (_ "Customer"))
+        ((eqv? owner-type GNC-OWNER-EMPLOYEE) (_ "Employee"))
+        ;; FALLTHROUGH
+        (else (_ "Vendor")))) 
+
 (define-macro (addto! alist element)
   `(set! ,alist (cons ,element ,alist)))
 
@@ -117,7 +151,7 @@
     (reverse heading-list)))
 
 
-(define num-buckets 4)
+(define num-buckets 5)
 (define (new-bucket-vector)
   (make-vector num-buckets (gnc-numeric-zero)))
 
@@ -128,8 +162,13 @@
     (set! begindate (decdate begindate ThirtyDayDelta))
     (gnc:make-date-list begindate to-date ThirtyDayDelta)))
 
+;; Have make-list create a stepped list, then add a date in the future for the "current" bucket 
+(define (make-extended-interval-list to-date) 
+    (define dayforcurrent (incdate to-date YearDelta)) ;; MAGIC CONSTANT 
+    (define oldintervalreversed (reverse (make-interval-list to-date)))          
+  (reverse (cons dayforcurrent oldintervalreversed))) 
 
-(define (make-aging-table options query bucket-intervals reverse?)
+(define (make-aging-table options query bucket-intervals reverse? date-type)
   (let ((lots (xaccQueryGetLots query QUERY-TXN-MATCH-ANY))
 	(buckets (new-bucket-vector))
 	(payments (gnc-numeric-zero))
@@ -161,21 +200,25 @@
      (lambda (lot)
        (let* ((bal (gnc-lot-get-balance lot))
 	      (invoice (gncInvoiceGetInvoiceFromLot lot))
-	      (post-date (gncInvoiceGetDatePosted invoice)))
-
+              (date (if (eq? date-type 'postdate)
+               (gncInvoiceGetDatePosted invoice) 
+               (gncInvoiceGetDateDue invoice)))
+              )
+         
 	 (if (not (gnc-numeric-zero-p bal))
 	     (begin
 	       (if reverse?
 		   (set! bal (gnc-numeric-neg bal)))
 	       (if (not (null? invoice))
 		   (begin
-		     (apply-invoice post-date bal))
+		     (apply-invoice date bal))
 		   (apply-payment bal))))))
      lots)
 
     (gnc:html-table-set-col-headers!
      table
-     (list (_ "0-30 days")
+     (list (_ "Current")
+           (_ "0-30 days")
 	   (_ "31-60 days")
 	   (_ "61-90 days")
 	   (_ "91+ days")))
@@ -288,7 +331,7 @@
     ))
 
 
-(define (make-txn-table options query acc start-date end-date)
+(define (make-txn-table options query acc start-date end-date date-type)
   (let ((txns (xaccQueryGetTransactions query QUERY-TXN-MATCH-ANY))
 	(used-columns (build-column-used options))
 	(total (gnc-numeric-zero))
@@ -342,14 +385,13 @@
 		    "total-number-cell"
 		    (gnc:make-gnc-monetary currency total)))))
 
-    (let* ((interval-vec (list->vector (make-interval-list end-date))))
+    (let* ((interval-vec (list->vector (make-extended-interval-list end-date))))
       (gnc:html-table-append-row/markup!
        table
        "grand-total"
-       (list (gnc:make-html-table-cell/size/markup
+       (list (gnc:make-html-table-cell/size
 	      1 (+ 1 (value-col used-columns))
-	      "centered-label-cell"
-	      (make-aging-table options query interval-vec reverse?)))))
+	      (make-aging-table options query interval-vec reverse? date-type)))))
 
     table))
 
@@ -367,7 +409,7 @@
    (gnc:make-simple-boolean-option "__reg" "reverse?" "" "" reverse?))
 
   (gnc:register-inv-option
-   (gnc:make-owner-option owner-page owner-string "v"
+   (gnc:make-owner-option owner-page (owner-string owner-type) "v"
 			  (N_ "The company for this report")
 			  (lambda () '()) #f owner-type))
 
@@ -416,13 +458,24 @@
   (gnc:register-inv-option
    (gnc:make-simple-boolean-option
     (N_ "Display Columns") amount-header
-    "hb" "Display the transaction amount?" #t))
+    "hb" (N_ "Display the transaction amount?") #t)) 
 
   (gnc:register-inv-option
    (gnc:make-string-option
     gnc:pagename-general (N_ "Today Date Format")
     "p" (N_ "The format for the date->string conversion for today's date.")
     (gnc-default-strftime-date-format)))
+  
+  (gnc:register-inv-option 
+   (gnc:make-multichoice-option 
+    gnc:pagename-general 
+    optname-date-driver 
+    "k" 
+    (N_ "Leading date") 
+    'duedate 
+    (list 
+     (vector 'duedate (N_ "Due Date") (N_ "Due date is leading")) ;; Should be using standard label for due date? 
+     (vector 'postdate (N_ "Post Date") (N_ "Post date is leading"))))) ;; Should be using standard label for post date? 
 
   (gnc:options-set-default-section gnc:*report-options* "General")
 
@@ -520,7 +573,7 @@
 
 (define (make-myname-table book date-format)
   (let* ((table (gnc:make-html-table))
-	 (slots (gnc-book-get-slots book))
+	 (slots (qof-book-get-slots book))
 	 (name (kvp-frame-get-slot-path-gslist
 		slots (append gnc:*kvp-option-path*
 			      (list gnc:*business-label* gnc:*company-name*))))
@@ -562,27 +615,20 @@
 	 (orders '())
 	 (query (qof-query-create-for-splits))
 	 (account (opt-val owner-page acct-string))
-	 (owner (opt-val owner-page owner-string))
-	 (start-date (gnc:timepair-start-day-time 
+         (start-date (gnc:timepair-start-day-time 
 		       (gnc:date-option-absolute-time
 			(opt-val gnc:pagename-general optname-from-date))))
 	 (end-date (gnc:timepair-end-day-time 
 		       (gnc:date-option-absolute-time
 			(opt-val gnc:pagename-general optname-to-date))))
 	 (book (gnc-get-current-book)) ;XXX Grab this from elsewhere
-	 (type (opt-val "__reg" "owner-type"))
-	 (type-str ""))
-
-    (cond
-      ((eqv? type GNC-OWNER-CUSTOMER)
-       (set! type-str (N_ "Customer")))
-      ((eqv? type GNC-OWNER-VENDOR)
-       (set! type-str (N_ "Vendor")))
-      ((eqv? type GNC-OWNER-EMPLOYEE)
-       (set! type-str (N_ "Employee"))))
+         (type (opt-val "__reg" "owner-type"))
+         (owner-descr (owner-string type))
+         (date-type (opt-val gnc:pagename-general optname-date-driver)) 
+	 (owner (opt-val owner-page owner-descr)))
 
     (gnc:html-document-set-title!
-     document (string-append (_ type-str) " " (_ "Report")))
+     document (string-append (doctype-str type) " " (_ "Report")))
 
     (if (gncOwnerIsValid owner)
 	(begin
@@ -590,12 +636,12 @@
 
 	  (gnc:html-document-set-title!
 	   document
-           (string-append (_ type-str ) " " (_ "Report:") " " (gncOwnerGetName owner)))
+           (string-append (doctype-str type) " " (_ "Report:") " " (gncOwnerGetName owner)))
 
            (gnc:html-document-set-headline!
             document (gnc:html-markup
                       "!" 
-                      (_ type-str )
+                      (doctype-str type)
                       " " (_ "Report:") " "
                       (gnc:html-markup-anchor
                        (gnc:owner-anchor-text owner)
@@ -604,7 +650,7 @@
 	  (if (not (null? account))
 	      (begin
 		(set! table (make-txn-table (gnc:report-options report-obj)
-					    query account start-date end-date))
+					    query account start-date end-date date-type))
 		(gnc:html-table-set-style!
 		 table "table"
 		 'attribute (list "border" 1)
@@ -644,10 +690,7 @@
 	(gnc:html-document-add-object!
 	 document
 	 (gnc:make-html-text
-	  (sprintf #f 
-		   (_ "No valid %s selected.  Click on the Options button to select a company.")
-		   (_ type-str))))) ;; FIXME because of translations: Please change this string into full sentences instead of sprintf, because in non-english languages the "no valid" has different forms depending on the grammatical gender of the "%s".
-
+	  (invalid-selection-string type)))) 
     (qof-query-destroy query)
     document))
 
@@ -688,6 +731,7 @@
 (gnc:define-report
  'version 1
  'name (N_ "Customer Report")
+ 'report-guid customer-report-guid
  'menu-path (list gnc:menuname-business-reports)
  'options-generator customer-options-generator
  'renderer reg-renderer
@@ -696,6 +740,7 @@
 (gnc:define-report
  'version 1
  'name (N_ "Vendor Report")
+ 'report-guid vendor-report-guid
  'menu-path (list gnc:menuname-business-reports)
  'options-generator vendor-options-generator
  'renderer reg-renderer
@@ -704,31 +749,32 @@
 (gnc:define-report
  'version 1
  'name (N_ "Employee Report")
+ 'report-guid employee-report-guid 
  'menu-path (list gnc:menuname-business-reports)
  'options-generator employee-options-generator
  'renderer reg-renderer
  'in-menu? #t)
 
-(define (owner-report-create-internal report-name owner account)
-  (let* ((options (gnc:make-report-options report-name))
-	 (owner-op (gnc:lookup-option options owner-page owner-string))
+(define (owner-report-create-internal report-guid owner account owner-type)
+  (let* ((options (gnc:make-report-options report-guid))
+	 (owner-op (gnc:lookup-option options owner-page (owner-string owner-type))) 
 	 (account-op (gnc:lookup-option options owner-page acct-string)))
 
     (gnc:option-set-value owner-op owner)
     (gnc:option-set-value account-op account)
-    (gnc:make-report report-name options)))
+    (gnc:make-report report-guid options)))
 
 (define (owner-report-create owner account)
   (let ((type (gncOwnerGetType (gncOwnerGetEndOwner owner))))
     (cond
       ((eqv? type GNC-OWNER-CUSTOMER)
-       (owner-report-create-internal (N_ "Customer Report") owner account))
+       (owner-report-create-internal customer-report-guid owner account type)) ;; Not sure whether to pass type, or to use the guid in the report function
 
       ((eqv? type GNC-OWNER-VENDOR)
-       (owner-report-create-internal (N_ "Vendor Report") owner account))
+       (owner-report-create-internal vendor-report-guid owner account type))
 
       ((eqv? type GNC-OWNER-EMPLOYEE)
-       (owner-report-create-internal (N_ "Employee Report") owner account))
+       (owner-report-create-internal employee-report-guid owner account type))
 
       (else #f))))
 
