@@ -17,6 +17,9 @@
 ;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 
+(use-modules (srfi srfi-13))
+(use-modules (gnucash printf))
+
 (define (list-ref-safe list elt)
   (if (> (length list) elt)
       (list-ref list elt)
@@ -94,7 +97,8 @@
                     ACCT-TYPE-CREDITLINE))
 	(cons ACCT-TYPE-EQUITY (list ACCT-TYPE-EQUITY))
 	(cons ACCT-TYPE-INCOME (list ACCT-TYPE-INCOME))
-	(cons ACCT-TYPE-EXPENSE (list ACCT-TYPE-EXPENSE)))))
+	(cons ACCT-TYPE-EXPENSE (list ACCT-TYPE-EXPENSE))
+	(cons ACCT-TYPE-TRADING (list ACCT-TYPE-TRADING)))))
 
 ;; Returns the name of the account type as a string, and in its plural
 ;; form (as opposed to xaccAccountGetTypeStr which gives the
@@ -118,7 +122,8 @@
     (cons ACCT-TYPE-MONEYMRKT (_ "Money Market"))
     (cons ACCT-TYPE-RECEIVABLE (_ "Accounts Receivable"))
     (cons ACCT-TYPE-PAYABLE (_ "Accounts Payable"))
-    (cons ACCT-TYPE-CREDITLINE (_ "Credit Lines")))
+    (cons ACCT-TYPE-CREDITLINE (_ "Credit Lines"))
+    (cons ACCT-TYPE-TRADING (_ "Trading Accounts")))
    type))
 
 ;; Get the list of all different commodities that are used within the
@@ -408,6 +413,17 @@
 	((list) commoditylist) ; this one is only for internal use
 	(else (gnc:warn "bad commodity-collector action: " action))))))
 
+(define (gnc:commodity-collector-get-negated collector)
+  (let
+    ((negated (gnc:make-commodity-collector)))
+    (negated 'minusmerge collector #f)
+    negated))
+
+(define (gnc:commodity-collectorlist-get-merged collectorlist)
+  (let
+    ((merged (gnc:make-commodity-collector)))
+    (for-each (lambda (collector) (merged 'merge collector #f)) collectorlist)
+    merged))
 
 ;; Bah. Let's get back to normal data types -- this procedure thingy
 ;; from above makes every code almost unreadable. First step: replace
@@ -613,39 +629,28 @@
 ;; the version which returns a commodity-collector
 (define (gnc:account-get-comm-balance-interval 
 	 account from to include-children?)
-  ;; Since this function calculates a balance difference it has to
-  ;; subtract the balance of the previous day's end (from-date)
-  ;; instead of the plain date.
-  (let ((this-collector (gnc:account-get-comm-balance-at-date 
-			 account to include-children?)))
-    (gnc-commodity-collector-minusmerge
-     this-collector
-     (gnc:account-get-comm-balance-at-date
-      account
-      (gnc:timepair-end-day-time (gnc:timepair-previous-day from))
-      include-children?))
-    this-collector))
+  (let ((account-list (if include-children?
+                          (let ((sub-accts (gnc-account-get-descendants-sorted account)))
+                            (if sub-accts
+                                (append (list account) sub-accts)
+                                (list account)))
+                          (list account))))
+    (gnc:account-get-trans-type-balance-interval account-list #f from to)))
 
 ;; This calculates the increase in the balance(s) of all accounts in
 ;; <accountlist> over the period from <from-date> to <to-date>.
 ;; Returns a commodity collector.
 (define (gnc:accountlist-get-comm-balance-interval accountlist from to)
-  (let ((collector (gnc:make-commodity-collector)))
-    (for-each (lambda (account)
-                (gnc-commodity-collector-merge
-                 collector (gnc:account-get-comm-balance-interval 
-                            account from to #f)))
-              accountlist)
-    collector))
+  (gnc:account-get-trans-type-balance-interval accountlist #f from to))
+
+(define (gnc:accountlist-get-comm-balance-interval-with-closing accountlist from to)
+  (gnc:account-get-trans-type-balance-interval-with-closing accountlist #f from to))
 
 (define (gnc:accountlist-get-comm-balance-at-date accountlist date)
-   (let ((collector (gnc:make-commodity-collector)))
-    (for-each (lambda (account)
-                (gnc-commodity-collector-merge
-                 collector (gnc:account-get-comm-balance-at-date 
-                            account date #f)))
-              accountlist)
-    collector))
+  (gnc:account-get-trans-type-balance-interval accountlist #f #f date))
+
+(define (gnc:accountlist-get-comm-balance-at-date-with-closing accountlist date)
+  (gnc:account-get-trans-type-balance-interval-with-closing accountlist #f #f date))
 
 ;; utility function - ensure that a query matches only non-voids.  Destructive.
 (define (gnc:query-set-match-non-voids-only! query book)
@@ -709,44 +714,49 @@
 
 ;; Sums up any splits of a certain type affecting a set of accounts.
 ;; the type is an alist '((str "match me") (cased #f) (regexp #f))
+;; If type is #f, sums all non-closing splits in the interval
 (define (gnc:account-get-trans-type-balance-interval
 	 account-list type start-date-tp end-date-tp)
-  (let* ((query (qof-query-create-for-splits))
-	 (splits #f)
-	 (get-val (lambda (alist key)
-		    (let ((lst (assoc-ref alist key)))
-		      (if lst (car lst) lst))))
-	 (matchstr (get-val type 'str))
-	 (case-sens (if (get-val type 'cased) #t #f))
-	 (regexp (if (get-val type 'regexp) #t #f))
-	 (total (gnc:make-commodity-collector))
-	 )
-    (qof-query-set-book query (gnc-get-current-book))
-    (gnc:query-set-match-non-voids-only! query (gnc-get-current-book))
-    (xaccQueryAddAccountMatch query account-list QOF-GUID-MATCH-ANY QOF-QUERY-AND)
-    (xaccQueryAddDateMatchTS
-     query
-     (and start-date-tp #t) start-date-tp
-     (and end-date-tp #t) end-date-tp QOF-QUERY-AND)
-    (xaccQueryAddDescriptionMatch
-     query matchstr case-sens regexp QOF-QUERY-AND)
-    
-    (set! splits (qof-query-run query))
+  (let* ((total (gnc:make-commodity-collector)))
     (map (lambda (split)
-		(let* ((shares (xaccSplitGetAmount split))
-		       (acct-comm (xaccAccountGetCommodity
-				   (xaccSplitGetAccount split)))
-		       )
-		  (gnc-commodity-collector-add total acct-comm shares)
-		  )
-		)
-	 splits
+           (let* ((shares (xaccSplitGetAmount split))
+                  (acct-comm (xaccAccountGetCommodity
+                              (xaccSplitGetAccount split)))
+                  (txn (xaccSplitGetParent split))
+                  )
+             (if type 
+                (gnc-commodity-collector-add total acct-comm shares)
+                (if (not (xaccTransGetIsClosingTxn txn))
+                    (gnc-commodity-collector-add total acct-comm shares)
+             )))
+           )
+	 (gnc:account-get-trans-type-splits-interval
+          account-list type start-date-tp end-date-tp)
 	 )
-    (qof-query-destroy query)
     total
     )
   )
 
+;; Sums up any splits of a certain type affecting a set of accounts.
+;; the type is an alist '((str "match me") (cased #f) (regexp #f))
+;; If type is #f, sums all splits in the interval (even closing splits)
+(define (gnc:account-get-trans-type-balance-interval-with-closing
+	 account-list type start-date-tp end-date-tp)
+  (let* ((total (gnc:make-commodity-collector)))
+    (map (lambda (split)
+           (let* ((shares (xaccSplitGetAmount split))
+                  (acct-comm (xaccAccountGetCommodity
+                              (xaccSplitGetAccount split)))
+                  )
+                (gnc-commodity-collector-add total acct-comm shares)
+             )
+           )
+	 (gnc:account-get-trans-type-splits-interval
+          account-list type start-date-tp end-date-tp)
+	 )
+    total
+    )
+  )
 ;; similar, but only counts transactions with non-negative shares and
 ;; *ignores* any closing entries
 (define (gnc:account-get-pos-trans-total-interval
@@ -809,6 +819,40 @@
     )
   )
 
+;; Return the splits that match an account list, date range, and (optionally) type
+;; where type is defined as an alist '((str "match me") (cased #f) (regexp #f))
+(define (gnc:account-get-trans-type-splits-interval
+         account-list type start-date-tp end-date-tp)
+  (if (null? account-list)
+      ;; No accounts given. Return empty list.
+      '()
+      ;; The normal case: There are accounts given.
+  (let* ((query (qof-query-create-for-splits))
+	 (splits #f)
+	 (get-val (lambda (alist key)
+		    (let ((lst (assoc-ref alist key)))
+		      (if lst (car lst) lst))))
+	 (matchstr (get-val type 'str))
+	 (case-sens (if (get-val type 'cased) #t #f))
+	 (regexp (if (get-val type 'regexp) #t #f))
+	 )
+    (qof-query-set-book query (gnc-get-current-book))
+    (gnc:query-set-match-non-voids-only! query (gnc-get-current-book))
+    (xaccQueryAddAccountMatch query account-list QOF-GUID-MATCH-ANY QOF-QUERY-AND)
+    (xaccQueryAddDateMatchTS
+     query
+     (and start-date-tp #t) start-date-tp
+     (and end-date-tp #t) end-date-tp QOF-QUERY-AND)
+    (if type (xaccQueryAddDescriptionMatch
+              query matchstr case-sens regexp QOF-QUERY-AND))
+    
+    (set! splits (qof-query-run query))
+    (qof-query-destroy query)
+    splits
+    )
+  )
+  )
+
 ;; utility to assist with double-column balance tables
 ;; a request is made with the <req> argument
 ;; <req> may currently be 'entry|'debit-q|'credit-q|'zero-q|'debit|'credit
@@ -855,3 +899,196 @@
     )
   )
 
+;; Returns the start date of the first period (period 0) of the budget.
+(define (gnc:budget-get-start-date budget)
+  (gnc-budget-get-period-start-date budget 0))
+
+;; Returns the end date of the last period of the budget.
+(define (gnc:budget-get-end-date budget)
+  (gnc-budget-get-period-end-date budget (- (gnc-budget-get-num-periods budget) 1)))
+
+
+(define (gnc:budget-accountlist-helper accountlist get-fn)
+  (let
+    (
+      (net (gnc:make-commodity-collector)))
+    (for-each
+      (lambda (account)
+        (net 'merge
+          (get-fn account)
+          #f))
+      accountlist)
+    net))
+
+;; Sums budget values for a single account from start-period (inclusive) to
+;; end-period (exclusive).
+;;
+;; start-period may be #f to specify the start of the budget
+;; end-period may be #f to specify the end of the budget
+;;
+;; Returns a commodity-collector.
+(define (gnc:budget-account-get-net budget account start-period end-period)
+  (if (not start-period) (set! start-period 0))
+  (if (not end-period) (set! end-period (gnc-budget-get-num-periods budget)))
+  (let*
+    (
+      (period start-period)
+      (net (gnc:make-commodity-collector))
+      (acct-comm (xaccAccountGetCommodity account)))
+    (while (< period end-period)
+      (net 'add acct-comm
+          (gnc-budget-get-account-period-value budget account period))
+      (set! period (+ period 1)))
+    net))
+
+;; Sums budget values for accounts in accountlist from start-period (inclusive)
+;; to end-period (exclusive).
+;;
+;; Note that budget values are never sign-reversed, so accountlist should
+;; contain only income accounts, only expense accounts, etc.  It would not be
+;; meaningful to include both income and expense accounts, or both asset and
+;; liability accounts.
+;;
+;; start-period may be #f to specify the start of the budget
+;; end-period may be #f to specify the end of the budget
+;;
+;; Returns a commodity-collector.
+(define (gnc:budget-accountlist-get-net budget accountlist start-period end-period)
+  (gnc:budget-accountlist-helper accountlist (lambda (account)
+    (gnc:budget-account-get-net budget account start-period end-period))))
+
+;; Finds the balance for an account at the start date of the budget.  The
+;; resulting balance is not sign-adjusted.
+;;
+;; Returns a commodity-collector.
+(define (gnc:budget-account-get-initial-balance budget account)
+  (gnc:account-get-comm-balance-at-date
+    account
+    (gnc:budget-get-start-date budget)
+    #f))
+
+;; Sums the balances of all accounts in accountlist at the start date of the
+;; budget.  The resulting balance is not sign-adjusted.
+;;
+;; Returns a commodity-collector.
+(define (gnc:budget-accountlist-get-initial-balance budget accountlist)
+  (gnc:budget-accountlist-helper accountlist (lambda (account)
+    (gnc:budget-account-get-initial-balance budget account))))
+
+;; Calculate the sum of all budgets of all children of an account for a specific period
+;;
+;; Parameters:
+;;   budget - budget to use
+;;   children - list of children
+;;   period - budget period to use
+;;
+;; Return value:
+;;   budget value to use for account for specified period.
+(define (budget-account-sum budget children period)
+  (let* ((sum
+           (cond
+             ((null? children) (gnc-numeric-zero))
+             (else
+               (gnc-numeric-add
+                 (gnc:get-account-period-rolledup-budget-value budget (car children) period)
+                 (budget-account-sum budget (cdr children) period)
+                 GNC-DENOM-AUTO GNC-RND-ROUND))
+               )
+        ))
+  sum)
+)
+
+;; Calculate the value to use for the budget of an account for a specific period.
+;; - If the account has a budget value set for the period, use it
+;; - If the account has children, use the sum of budget values for the children
+;; - Otherwise, use 0.
+;;
+;; Parameters:
+;;   budget - budget to use
+;;   acct - account
+;;   period - budget period to use
+;;
+;; Return value:
+;;   sum of all budgets for list of children for specified period.
+(define (gnc:get-account-period-rolledup-budget-value budget acct period)
+  (let* ((bgt-set? (gnc-budget-is-account-period-value-set budget acct period))
+        (children (gnc-account-get-children acct))
+        (amount (cond
+                  (bgt-set? (gnc-budget-get-account-period-value budget acct period))
+          ((not (null? children)) (budget-account-sum budget children period))
+          (else (gnc-numeric-zero)))
+        ))
+  amount)
+)
+
+;; Sums rolled-up budget values for a single account from start-period (inclusive) to
+;; end-period (exclusive).
+;; - If the account has a budget value set for the period, use it
+;; - If the account has children, use the sum of budget values for the children
+;; - Otherwise, use 0.
+;;
+;; start-period may be #f to specify the start of the budget
+;; end-period may be #f to specify the end of the budget
+;;
+;; Returns a gnc-numeric value
+(define (gnc:budget-account-get-rolledup-net budget account start-period end-period)
+  (if (not start-period) (set! start-period 0))
+  (if (not end-period) (set! end-period (gnc-budget-get-num-periods budget)))
+  (let*
+    (
+      (period start-period)
+      (net (gnc-numeric-zero))
+      (acct-comm (xaccAccountGetCommodity account)))
+    (while (< period end-period)
+      (set! net (gnc-numeric-add net
+          (gnc:get-account-period-rolledup-budget-value budget account period)
+	  GNC-DENOM-AUTO GNC-RND-ROUND))
+      (set! period (+ period 1)))
+    net))
+
+(define (gnc:get-assoc-account-balances accounts get-balance-fn)
+  (let*
+    (
+      (initial-balances (list)))
+    (for-each
+      (lambda (account)
+        (set! initial-balances
+          (append initial-balances
+            (list (list account (get-balance-fn account))))))
+      accounts)
+    initial-balances))
+
+(define (gnc:select-assoc-account-balance account-balances account)
+  (let*
+    (
+      (account-balance (car account-balances))
+      (result
+        (if
+          (equal? account-balance '())
+          #f
+          (if
+            (equal? (car account-balance) account)
+            (car (cdr account-balance))
+            (gnc:select-assoc-account-balance
+              (cdr account-balances)
+              account)))))
+    result))
+
+(define (gnc:get-assoc-account-balances-total account-balances)
+  (let
+    (
+      (total (gnc:make-commodity-collector)))
+    (for-each
+      (lambda (account-balance)
+        (total 'merge (car (cdr account-balance)) #f))
+      account-balances)
+    total))
+
+;; Adds "file:///" to the beginning of a URL if it doesn't already exist
+;;
+;; @param url URL
+;; @return URL with "file:///" as the URL type if it isn't already there
+(define (make-file-url url)
+  (if (string-prefix? "file:///" url)
+     url
+     (string-append "file:///" url)))
